/*
 * Copyright (c) 1979 Regents of the University of California
 * C shell
 * Definitions to avoid problems with multiply defined symbols
 * from loader.  Variables come from sh.h
 *
 * @(#)defs.c	3.1 of 5/22/85.
 */

static char sccsid[] = "@(#)defs.c	3.1";

#include "sh.h"

/*
 * Global flags
 */
bool	didcch;		/* Have closed unused fd's for child */
bool	didfds;		/* Have setup i/o fd's for child */
bool	doneinp;	/* EOF indicator after reset from readc */
bool	exiterr;	/* Exit if error or non-zero exit status */
bool	child;		/* Child shell ... errors cause exit */
bool	haderr;		/* Reset was because of an error */
bool	intty;		/* Input is a tty */
bool	intact;		/* We are interactive... therefore prompt */
bool	justpr;		/* Just print because of :p hist mod */
bool	loginsh;	/* We are a loginsh -> .login/.logout */
bool	noexec;		/* Don't execute, just syntax check */
bool	setintr;	/* Set interrupts on/off -> Wait intr... */
bool	timflg;		/* Time the next waited for command */

/*
 * Global i/o info
 */
char	*arginp;	/* Argument input for sh -c and internal `xx`*/
int	onelflg;	/* 2 -> need line for -t, 1 -> exit on read */
char	*file;		/* Name of shell file for $0 */

char	*err;		/* Error message from scanner/parser */
char	*shtemp;	/* Temp name for << shell files in /tmp */
time_t	time0;		/* Time at which the shell started */

/*
 * Miscellany
 */
char	*doldol;	/* Character pid for $$ */
int	uid;		/* Invokers uid */
time_t	chktim;		/* Time mail last checked */

/*
 * To be able to redirect i/o for builtins easily, the shell moves the i/o
 * descriptors it uses away from 0,1,2.
 * Ideally these should be in units which are closed across exec's
 * (this saves work) but for version 6, this is not usually possible.
 * The desired initial values for these descriptors are defined in
 * sh.local.h.
 */
short	SHIN;		/* Current shell input (script) */
short	SHOUT;		/* Shell output */
short	SHDIAG;		/* Diagnostic output... shell errs go here */
short	OLDSTD;		/* Old standard input (def for cmds) */

/*
 * Error control
 *
 * Errors in scanning and parsing set up an error message to be printed
 * at the end and complete.  Other errors always cause a reset.
 * Because of source commands and .cshrc we need nested error catches.
 */

jmp_buf	reslab;

char	*gointr;	/* Label for an onintr transfer */
int	(*parintr)();	/* Parents interrupt catch */
int	(*parterm)();	/* Parents terminate catch */

/*
 * Each level of input has a buffered input structure.
 * There are one or more blocks of buffered input for each level,
 * exactly one if the input is seekable and tell is available.
 * In other cases, the shell buffers enough blocks to keep all loops
 * in the buffer.
 */
struct	Bin B;

/*
 * The shell finds commands in loops by reseeking the input
 * For whiles, in particular, it reseeks to the beginning of the
 * line the while was on; hence the while placement restrictions.
 */
off_t	lineloc;

#ifdef	TELL
bool	cantell;		/* Is current source tellable ? */
#endif


/*
 * Labuf implements a general buffer for lookahead during lexical operations.
 * Text which is to be placed in the input stream can be stuck here.
 * We stick parsed ahead $ constructs during initial input,
 * process id's from `$$', and modified variable values (from qualifiers
 * during expansion in sh.dol.c) here.
 */
char	labuf[256];
char	*lap;

/*
 * Structure defining the existing while/foreach loops at this
 * source level.  Loops are implemented by seeking back in the
 * input.  For foreach (fe), the word list is attached here.
 */
struct	whyle *whyles;

/*
 * Variable structure
 *
 * Lists of aliases and variables are sorted alphabetically by name
 */
struct	varent shvhed, aliases;

/*
 * The following are for interfacing redo substitution in
 * aliases to the lexical routines.
 */
struct	wordent *alhistp;	/* Argument list (first) */
struct	wordent *alhistt;	/* Node after last in arg list */
char	**alvec;		/* The (remnants of) alias vector */

/*
 * Filename/command name expansion variables
 */
short	gflag;			/* After tglob ->is globbing needed? */

/*
 * Variables for filename expansion
 */
char	**gargv;		/* Pointer to the (stack) arglist */
short	gargc;			/* Number args in gargv */
short	gnleft;

/*
 * Variables for command expansion.
 */
char	**pargv;		/* Pointer to the argv list space */
char	*pargs;			/* Pointer to start current word */
short	pargc;			/* Count of arguments in pargv */
short	pnleft;			/* Number of chars left in pargs */
char	*pargcp;		/* Current index into pargs */

/*
 * History list
 *
 * Each history list entry contains an embedded wordlist
 * from the scanner, a number for the event, and a reference count
 * to aid in discarding old entries.
 *
 * Essentially "invisible" entries are put on the history list
 * when history substitution includes modifiers, and thrown away
 * at the next discarding since their event numbers are very negative.
 */
struct	Hist Histlist;
struct	wordent	paraml;		/* Current lexical word list */
int	eventno;		/* Next events number */
int	lastev;			/* Last event reference (default) */
char	*bname;
char	HIST, HISTSUB;		/* History substutute characters */
